import os
from os.path import isdir, join
from Globals import package_home
from Products.CMFCore.utils import minimalpath
from Products.CMFCore.DirectoryView import manage_listAvailableDirectories

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from cStringIO import StringIO
import string

from Products.CMFPhoto.config import PROJECTNAME, GLOBALS, SKINS_DIR, SKINNAME 
from Products.CMFPhoto import Photo

skinname = SKINNAME
photo_globals = GLOBALS

def install_skins(self, out, globals=GLOBALS, product_skins_dir=SKINS_DIR):
    out.write('CMFPhoto Installation on %s\n' % self.id)
    out.write('======================\n\n')

    # Setup the skins
    skinsTool = getToolByName(self, 'portal_skins')

    fullProductSkinsPath = join(package_home(globals), product_skins_dir)
    productSkinsPath = minimalpath(fullProductSkinsPath)
    registered_directories = manage_listAvailableDirectories()
    if productSkinsPath+'/'+skinname not in registered_directories:
        registerDirectory(product_skins_dir+skinname, globals)
    try:
        addDirectoryViews(skinsTool, product_skins_dir, globals)
        out.write("Added %s directory view to portal_skins\n" % skinname)
    except:
        pass  # directory view has already been added
        out.write("%s directory view already existed in portal_skins\n" % skinname)

    # Insert 'photo_content' into the skin configurations.
    files = os.listdir(fullProductSkinsPath)
    for productSkinName in files:
        if (isdir(join(fullProductSkinsPath, productSkinName))
            and productSkinName != 'themes'
            and productSkinName != 'CVS'
            and productSkinName != '.svn'):
            for skinName in skinsTool.getSkinSelections():
                path = skinsTool.getSkinPath(skinName)
                path = [i.strip() for i in  path.split(',')]
                try:
                    if productSkinName not in path:
                        path.insert(path.index('custom') +1, productSkinName)
                except ValueError:
                    if productSkinName not in path:
                        path.append(productSkinName)
                path = ','.join(path)

                # addSkinSelection will replace existing skins as well.
                skinsTool.addSkinSelection(skinName, path)
                out.write("Added %s to %s skin\n" % (skinname, skinName))
    path = [p.strip() for p in skinsTool.getSkinPath('Plone Default').split(',')]
    path.remove('custom')

def register_types(self, out):
    typesTool = getToolByName(self, 'portal_types')

    for f in (Photo.factory_type_information, ):
        if f['id'] not in typesTool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), f)
            typesTool._setObject(f['id'], cfm)
            out.write('Registered with the types tool\n')
        else:
            out.write('Object "%s" already existed in the types tool\n' % (f['id']))


def removeSkin(self, out):
    skinsTool = getToolByName(self, 'portal_skins')

    skins = skinsTool.getSkinSelections()
    for skin in skins:
        path = skinsTool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if skinname in path:
            path.remove(skinname)
            path = string.join(path, ', ')
            skinsTool.addSkinSelection(skin, path)

    if skinname in skinsTool.objectIds():
        skinsTool._delObject(skinname)
        out.write('Removed %s from the portal_skins\n' % skinname)

def remove_types(self, out):
    typesTool = getToolByName(self, 'portal_types')

    for f in (Photo.factory_type_information, ):
        if f['id'] not in typesTool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), f)
            typesTool._delObject(f['id'], cfm)
            out.write('Removed type Photo from portal_types\n')

def install(self):
    out = StringIO()

    install_skins (self, out)

    # try for plone
    try:
        import Products.CMFPlone
    except ImportError:
        pass
    else:
        register_types (self, out)

    out.write ('%s Installation completed.\n' % PROJECTNAME)
    return out.getvalue()

def uninstall (self):
    out = StringIO ()

    removeSkin (self, out)
    remove_types(self, out)
    out.write ('Uninstallation completed.\n')
    return out.getvalue()

